==> timestream-0.6.19.tar.gz: timestream-0.6.19/timestream/batch.py <==
import logging
from timestream.dimensions import TimestreamDimensions
from timestream.record import TimestreamRecord
from timestream import DEFAULT_LOGGER
from typing import List, Tuple

DEFAULT_MAX_BATCH_SIZE = 100 # maximum set by AWS Timestream service limits
DEFAULT_MIN_COMMON_DIMENSIONS = 3


class Batch:
    def __init__(
        self,
        records: List[TimestreamRecord] = None,
        common_dimensions: TimestreamDimensions = None
    ):
        self.records = records or []
        self.common_dimensions = common_dimensions or TimestreamDimensions()
        


def batch_records(
    records: List[TimestreamRecord],
    max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
    min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS
) -> List[Batch]:
    batches, remaining = [], records
    DEFAULT_LOGGER.info(f"MAX BATCH || {max_batch_size}  MIN COMMON DIM || {min_common_dimensions}")
    while len(remaining) > 0:
        DEFAULT_LOGGER.info(f"Remaining records to batch || {len(remaining)}")
        batch, remaining = _batch_records_helper(
            remaining, max_batch_size, min_common_dimensions
        )
        batches.append(batch)
    return batches


def _batch_records_helper(
    records: List[TimestreamRecord],
    max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
    min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS
) -> Tuple[Batch, list]:
    max_batch_size = max_batch_size if not max_batch_size == None else DEFAULT_MAX_BATCH_SIZE
    min_common_dimensions = min_common_dimensions if not min_common_dimensions == None else DEFAULT_MIN_COMMON_DIMENSIONS
    DEFAULT_LOGGER.info(f"HELPER MAX BATCH || {max_batch_size}  MIN COMMON DIM || {min_common_dimensions}")
    
    assert max_batch_size > 0 and max_batch_size <= DEFAULT_MAX_BATCH_SIZE
    assert min_common_dimensions >= 0

    if len(records) < 2:
        return Batch(records, TimestreamDimensions()), []
    
    indexes = [0] # indexes for all the records in the batch
    common_dimensions = records[0].dimensions
    min_common_dimensions = min(min_common_dimensions, len(common_dimensions))
    
    for i in range(1, len(records)):
        if len(indexes) >= max_batch_size:
            break

        dimensions, shared = records[i].dimensions, {}
        for k in common_dimensions.get_dimension_keys():
            if dimensions.get_dimension(k) == common_dimensions.get_dimension(k):
                shared[k] = True
        # Add the record to the batch if it has the minimum number of shared dimensions
        if len(shared) >= min_common_dimensions:
            keys_to_remove = [k for k in common_dimensions.get_dimension_keys() if k not in shared]
            for k in keys_to_remove:
                common_dimensions.remove_dimension(k)
            indexes.append(i)

    batch = Batch([records[i] for i in indexes], common_dimensions)
    remaining = [records[i] for i in range(len(records)) if i not in indexes]
    return batch, remaining

==> timestream-0.7.998.tar.gz: timestream-0.7.998/timestream/batch.py <==
from timestream.dimensions import TimestreamDimensions
from timestream.record import TimestreamRecord
from timestream import DEFAULT_LOGGER
from typing import List, Tuple

DEFAULT_MAX_BATCH_SIZE = 100 # maximum set by AWS Timestream service limits
DEFAULT_MIN_COMMON_DIMENSIONS = 3


class Batch:
    def __init__(
        self,
        records: List[TimestreamRecord] = None,
        common_dimensions: TimestreamDimensions = None
    ):
        self.records = records or []
        self.common_dimensions = common_dimensions or TimestreamDimensions()
        


def batch_records(
    records: List[TimestreamRecord],
    max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
    min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS
) -> List[Batch]:
    if not min_common_dimensions:
        return dimensionless_batch(records, max_batch_size)

    batches, remaining = [], records
    DEFAULT_LOGGER.info(f"MAX BATCH || {max_batch_size}  MIN COMMON DIM || {min_common_dimensions}")
    while len(remaining) > 0:
        DEFAULT_LOGGER.info(f"Remaining records to batch || {len(remaining)}")
        batch, remaining = _batch_records_helper(
            remaining, max_batch_size, min_common_dimensions
        )
        batches.append(batch)
    return batches


def _batch_records_helper(
    records: List[TimestreamRecord],
    max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
    min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS
) -> Tuple[Batch, list]:
    max_batch_size = max_batch_size if not max_batch_size == None else DEFAULT_MAX_BATCH_SIZE
    min_common_dimensions = min_common_dimensions if not min_common_dimensions == None else DEFAULT_MIN_COMMON_DIMENSIONS
    DEFAULT_LOGGER.info(f"HELPER MAX BATCH || {max_batch_size}  MIN COMMON DIM || {min_common_dimensions}")
    
    assert max_batch_size > 0 and max_batch_size <= DEFAULT_MAX_BATCH_SIZE
    assert min_common_dimensions >= 0

    if len(records) < 2:
        return Batch(records, TimestreamDimensions()), []
    
    indexes = [0] # indexes for all the records in the batch
    common_dimensions = records[0].dimensions
    min_common_dimensions = min(min_common_dimensions, len(common_dimensions))
    
    for i in range(1, len(records)):
        if len(indexes) >= max_batch_size:
            break

        dimensions, shared = records[i].dimensions, {}
        for k in common_dimensions.get_dimension_keys():
            if dimensions.get_dimension(k) == common_dimensions.get_dimension(k):
                shared[k] = True
        # Add the record to the batch if it has the minimum number of shared dimensions
        if len(shared) >= min_common_dimensions:
            keys_to_remove = [k for k in common_dimensions.get_dimension_keys() if k not in shared]
            for k in keys_to_remove:
                common_dimensions.remove_dimension(k)
            indexes.append(i)

    batch = Batch([records[i] for i in indexes], common_dimensions)
    remaining = [records[i] for i in range(len(records)) if i not in indexes]
    return batch, remaining


def dimensionless_batch(
    records: List[TimestreamRecord],
    max_batch_size: int = DEFAULT_MAX_BATCH_SIZE
) -> List[Batch]:
    '''Batch records in groups of max_batch_size ignoring dimensions and dimension commonalities'''
    max_batch_size = max_batch_size or DEFAULT_MAX_BATCH_SIZE
    assert isinstance(max_batch_size, int), f'Invalid max batch size: {max_batch_size}'
    batches = []
    for i in range(0, len(records), max_batch_size):
        batches.append(Batch(
            records=records[i:i+max_batch_size])
        )
    return batches
